'===========================================================
' FILEHUNT (FILEHUNT.BAS)
'
' Copyright (C) 1995 Jonathan Waldman
'
' Written for PC Magazine, July 1995, US Edition
'
' Revision History:
'
' 1.0   Initial Release
'
'===========================================================

Option Explicit

' these API calls are used to create a horizontal scroll bar
' for both the list box and directory list box controls
Declare Function SendMessage& Lib "user" (ByVal hWnd%, ByVal wMsg%, ByVal wParam%, ByVal lParam&)
Declare Function GetTempFileName Lib "Kernel" (ByVal cDriveLetter As Integer, ByVal lpPrefixString As String, ByVal wUnique As Integer, ByVal lptempfilename As String) As Integer
Declare Function GetScrollPos Lib "User" (ByVal hWnd As Integer, ByVal nBar As Integer) As Integer

Global Const WM_HSCROLL = &H114
Global Const SB_THUMBPOSITION = 4

' these constants are used to create a horizontal scroll bar on the list box control
Global Const LB_SETHORIZONTALEXTENT = &H400 + 21
Global Const NUL = 0
Global Const SB_HORZ = 0

' application name
Global Const APP_NAME = "File Hunter 1.00"

Global Const PATH_NAME_SIZE% = 66      '64 + 2 for drive and colon
Global Const LOGO_CYCLE_NO = 20
Global Const DELIM_CHAR = ";"
Global Const ANIMATE_DELAY = .75
Global Const MAX_FIND_LEN = 65535
Global Const MAX_LB_ENTRIES = 5440
Global Const HIT_LIST_MAX_LEN = 128
Global Const DIR_DISPLAY_LEN = 36

' condition constants
Global Const CONDITION_ON = 1
Global Const CONDITION_AFTER = 2
Global Const CONDITION_BEFORE = 3
Global Const CONDITION_BETWEEN = 4
Global Const CONDITION_ONORAFTER = 5
Global Const CONDITION_ONORBEFORE = 6
Global Const CONDITION_NOTON = 7
Global Const CONDITION_NOTBETWEEN = 8

' Hit List structure
Type HitListRecord
   FileName As String * 12
   Size As Long
   DateTime As String * 19
   Attributes As Integer
   ' used as a directory name pointer into another TEMP File
   DirectoryNo As Long
End Type

' global settings structure
Type FilterInfo
   ViewOption As Integer
   IncludeSudDirs As Integer
   ReadOnly As Integer
   Hidden As Integer
   System As Integer
   Archive As Integer
   ApplyDate As Integer
   ApplyTime As Integer
   ApplySize As Integer
   Apply2Size As Integer
   ConditionDate As Integer
   ConditionDateLI As Integer
   ConditionTime As Integer
   ConditionTimeLI As Integer
   ConditionSize As Integer
   ConditionSizeLI As Integer
   ValueDate1 As Variant
   ValueDate2 As Variant
   ValueTime1 As Variant
   ValueTime2 As Variant
   ValueSize1 As Long
   ValueSize2 As Long
End Type

' global settings structure variable
Global gFilters As FilterInfo

' used as a handle to the temporary hit list random file
Global fHitListFileHandle As Integer
' original path
Global gOriginalPath As String * PATH_NAME_SIZE
' animation vars
Global gCycleCounterInc%
Global gCycleCounter%
' cancelled flag
Global gSearchCancelled%
Global gDirsArray$()

Sub AddLBItem (StringToAdd$, SourceC As Control, DestC As Control, Filter%)

   Dim i%, TotalDirSearchByte%

   ' adds an item from one list box to another
   StringToAdd$ = Trim$(UCase$(StringToAdd$))
   If Filter% Then
      StringToAdd$ = GetItemInParentheses(StringToAdd$)
   End If
   If Len(StringToAdd$) Then
      For i% = 0 To DestC.ListCount - 1
	 If DestC.List(i%) = StringToAdd$ Then
	    MsgBox "You've already added this item.", MB_ICONEXCLAMATION, APP_NAME & " - Error"
	    StringToAdd$ = ""
	    Exit For
	 End If
	 TotalDirSearchByte% = TotalDirSearchByte% + Len(Trim$(SourceC.List(SourceC.ListIndex))) + 1
      Next i%
      If StringToAdd$ <> "" Then
	 If (TotalDirSearchByte% < MAX_FIND_LEN) And (DestC.ListCount < 5441) Then
	    DestC.AddItem StringToAdd$
	 Else
	    MsgBox "You cannot add the highlighted item because the search list will be greater than the maximum of " & Trim$(Str$(MAX_FIND_LEN)) & " bytes or the list box will contain greater than " & Trim$(Str$(MAX_LB_ENTRIES)) & " entries.", MB_ICONEXCLAMATION, APP_NAME & " - Error"
	 End If
      End If
   End If
   
End Sub

Function AttrString (Attr) As String

   Dim Work$

   ' creates the attribute string for the Hit List display
   Work$ = IIf(Attr And ATTR_READONLY, "R", "-")
   Work$ = Work$ & IIf(Attr And ATTR_HIDDEN, "H", "-")
   Work$ = Work$ & IIf(Attr And ATTR_SYSTEM, "S", "-")
   Work$ = Work$ & IIf(Attr And ATTR_ARCHIVE, "A", "-")

   AttrString = Work$

End Function

Function CheckDirectories (D As Control) As Integer

   On Error Resume Next
   Dim ParseVal%, ret%
   ReDim gDirsArray$(0)

   ret% = Parse((frmMain!txtDrivePaths.Text), gDirsArray$(), True)
   If ret% = 0 Or ret% = -2 Then
      ' attempts to validate specified directories
      CheckDirectories = True
      For ParseVal% = 0 To UBound(gDirsArray$, 1)
	 If Len(gDirsArray$(ParseVal%)) > 3 Then
	    If Right$(gDirsArray$(ParseVal%), 1) = "\" Then
	       gDirsArray$(ParseVal%) = Left$(gDirsArray$(ParseVal%), Len(gDirsArray$(ParseVal%)) - 1)
	    End If
	 End If
	 If Len(gDirsArray$(ParseVal%)) = 2 Then
	    gDirsArray$(ParseVal%) = gDirsArray$(ParseVal%) & "\"
	 End If
	 D.Path = gDirsArray$(ParseVal%)
	 If Err = 76 Then
	    Screen.MousePointer = DEFAULT
	    MsgBox "The directory " & Chr$(34) & gDirsArray$(ParseVal%) & Chr$(34) & " could not be accessed.", MB_OK Or MB_ICONSTOP, ""
	    CheckDirectories = False
	    gSearchCancelled = -2
	    Exit For
	 End If
	 If Err = 68 Then
	    Screen.MousePointer = DEFAULT
	    MsgBox "A drive you specified (" & Chr$(34) & gDirsArray$(ParseVal%) & Chr$(34) & ") could not be accessed. Please correct this problem before accessing other File Hunter features.", MB_OK Or MB_ICONSTOP, ""
	    CheckDirectories = False
	    gSearchCancelled = -2
	    Exit For
	 End If
      Next ParseVal%
   Else
      If ret% = -2 Then
      End If
   End If
   
End Function

Sub CheckToggle3State (C As Control, OldValue)

   ' creates a three-state check box
   C.Tag = 1
   OldValue = OldValue + 1
   If OldValue > 2 Then OldValue = 0
   C.Value = OldValue
   C.Tag = 0

End Sub

Function FileCreateDate$ (CreateDate$)

   Dim DateStr$, TimeStr$, lenStr, TrimmedStr$

   lenStr = Len(Trim$(CreateDate$))
   TrimmedStr$ = Trim$(CreateDate$)
   If lenStr < 8 Then
      If lenStr < 8 Then
	 CreateDate$ = TrimmedStr$ & Space$(8 - lenStr) & "12:00:00 AM"
      Else
	 CreateDate$ = TrimmedStr$ & "12:00:00 AM"
      End If
   End If
   ' gets the file creation date and time
   DateStr$ = Format$(Left$(CreateDate$, InStr(1, CreateDate$, " ") - 1), "mm-dd-yyyy")
   TimeStr$ = Format$(Right$(CreateDate$, Len(CreateDate$) - InStr(1, CreateDate$, " ") + 1), "hh:mm:ss AM/PM")
   ' the time of 12:00:00 AM needs to be appended to the TimeStr$ variable because
   ' the FileDateTime function returns a null value when the time stamp on a file
   ' is midnight
   If Trim$(TimeStr$) = "" Then
      TimeStr$ = "12:00:00 AM"
   End If
   FileCreateDate$ = DateStr$ & "  " & TimeStr$

End Function

Sub FillFileMasks (C As Control)

   ' pre-defined file masks
   C.AddItem "All Files (*.*)"
   C.AddItem "Program Files (*.EXE;*.COM;*.BAT)"
   C.AddItem "Dynamic Link Libraries (*.DLL)"
   C.AddItem "Backup Files (*.BAK;*.BK?)"
   C.AddItem "Program Information Files (*.PIF)"
   C.AddItem "Initialization Files (*.INI)"
   C.AddItem "Text Files (*.TXT;*.ASC)"
   C.AddItem "Document Files (*.DOC;*.WP)"
   C.AddItem "Spreadsheet Files (*.WK1;*.WK2;*.WK3;*.WKS;*.XLS)"
   C.AddItem "Temp Files (~*.*)"
   C.AddItem "Borland Backup Files (*.~*)"
   C.AddItem "ZIP Files (*.ZIP)"
   
End Sub

Function GetItemInParentheses (Work$) As String

   ' get the item in parentheses for the pre-defined
   ' file masks
   If InStr(Work$, "(") <> 0 And InStr(Work$, ")") <> 0 Then
     GetItemInParentheses$ = Mid$(Work$, InStr(Work$, "(") + 1, Len(Work$) - InStr(Work$, "(") - 1)
   Else
     GetItemInParentheses$ = Work$
   End If

End Function

Function GetTempFile$ ()

   Dim lptempfilename$, ret%

   ' generate a TEMP file name for the virtual list box
   lptempfilename$ = Space$(144)
   ret% = GetTempFileName(ByVal 0, ByVal "PCM", ByVal 0, ByVal lptempfilename$)
   GetTempFile$ = Mid$(lptempfilename$, 1, Len(Trim$(lptempfilename$)) - 1)

End Function

Function KeepMeAnalysis (ConditionCode%, Work As Variant, Work2 As Variant, WorkCompare As Variant) As Integer

   ' determines if the current file should be added to the
   ' Hit List based on date/time/size filters
   Select Case ConditionCode%
      Case CONDITION_ON
	 KeepMeAnalysis% = (WorkCompare = Work)
      Case CONDITION_AFTER
	 KeepMeAnalysis% = (WorkCompare > Work)
      Case CONDITION_BEFORE
	 KeepMeAnalysis% = (WorkCompare < Work)
      Case CONDITION_BETWEEN
	 KeepMeAnalysis% = (WorkCompare >= Work) And (WorkCompare <= Work2)
      Case CONDITION_ONORAFTER
	 KeepMeAnalysis% = (WorkCompare >= Work)
      Case CONDITION_ONORBEFORE
	 KeepMeAnalysis% = (WorkCompare <= Work)
      Case CONDITION_NOTON
	 KeepMeAnalysis% = (WorkCompare <> Work)
      Case CONDITION_NOTBETWEEN
	 KeepMeAnalysis% = Not ((WorkCompare >= Work) And (WorkCompare <= Work2))
   End Select

End Function

Sub LoadDefaults ()

   ' default Advanced Screen settings
   gFilters.ViewOption = -1
   gFilters.IncludeSudDirs = -1
   gFilters.ReadOnly = 2
   gFilters.Hidden = 2
   gFilters.System = 2
   gFilters.Archive = 2
   gFilters.ApplyDate = 0
   gFilters.ApplyTime = 0
   gFilters.ApplySize = 0
   gFilters.ConditionDateLI = 1
   gFilters.ConditionTimeLI = 1
   gFilters.ConditionSizeLI = 1
   gFilters.ValueDate1 = ""
   gFilters.ValueDate2 = ""
   gFilters.ValueTime1 = ""
   gFilters.ValueTime2 = ""
   gFilters.ValueSize1 = 0
   gFilters.ValueSize2 = 0

End Sub

Sub Main ()

   App.Title = "File Hunter 1.00"
   App.HelpFile = "FILEHUNT.HLP"
   LoadDefaults
   frmMain.Show MODAL

End Sub

Function Parse (Work$, Parsed$(), NoDuplicatesFlag%) As Integer

   Dim WorkOffset%, WorkPart$, WorkPartNo%, WorkDelimLoc%
   ReDim Parsed$(0)

   ' parses delimited items in a list and places contents into
   ' an array
   Parse = -1
   
   If Work$ = "" Then
      Parse = -2
      Exit Function
   End If

   Work$ = PreParse(Work$)

   WorkPartNo% = 1
   WorkOffset% = 1
   WorkDelimLoc% = InStr(1, Work$, DELIM_CHAR)
   If WorkDelimLoc% > 0 Then
      WorkPart$ = Trim$(Mid$(Work$, WorkOffset%, WorkDelimLoc% - WorkOffset%))
      Do
	 If WorkOffset% > 1 Then
	    ReDim Preserve Parsed$((UBound(Parsed$, 1)) + 1)
	 End If
	 Parse = UBound(Parsed)
	 Parsed$(UBound(Parsed)) = WorkPart$
	 If WorkDelimLoc% < Len(Work$) Then
	    WorkOffset% = WorkDelimLoc% + 1
	    WorkDelimLoc% = InStr(WorkDelimLoc% + 1, Work$, DELIM_CHAR)
	    If WorkDelimLoc% = 0 Then WorkDelimLoc% = Len(Work$) + 1
	    WorkPart$ = Mid$(Work$, WorkOffset%, WorkDelimLoc% - WorkOffset%)
	 Else
	    Exit Do
	 End If
      Loop
   Else
      Parsed$(0) = Work$
   End If

   If NoDuplicatesFlag% Then
      Parse = Not (PostParse(Parsed$()))
   End If

End Function

Function PostParse (Parsed$())

   Dim i%, ParseVal%

   ' validates integrity of parsed items
   PostParse = True
   For ParseVal% = 0 To UBound(Parsed$, 1)
      For i% = (ParseVal% + 1) To UBound(Parsed$, 1)
	 If Parsed$(ParseVal%) = Parsed$(i%) Then
	    Screen.MousePointer = DEFAULT
	    MsgBox "A delimited item " & Chr$(34) & IIf(Len(Parsed$(ParseVal%)) = 0, "{NULL}", Parsed$(ParseVal%)) & Chr$(34) & " was specified more than once.", MB_OK Or MB_ICONSTOP, ""
	    PostParse = False
	    Exit Function
	 End If
      Next i%
   Next ParseVal%

End Function

Function PreParse (Work$) As String

   ' trims delimiters before delimited string is parsed
   Do
      If Right$(Work$, 1) <> DELIM_CHAR Then
	 Exit Do
      End If
      Work$ = Left$(Work$, Len(Work$) - 1)
   Loop
   Do
      If Left$(Work$, 1) <> DELIM_CHAR Then
	 Exit Do
      End If
      Work$ = Mid$(Work$, 2)
   Loop

   PreParse = Work$

End Function

Sub RemoveLBItem (C As Control)

   ' removes an item from a list box
   Dim X%
   If C.ListIndex >= 0 Then
      X% = C.ListIndex
      C.RemoveItem X%
      If C.ListCount Then
	 If X% < C.ListCount Then
	    C.ListIndex = X%
	 Else
	    C.ListIndex = C.ListCount - 1
	 End If
      End If
   End If

End Sub

Function TrimDelimChar (S As String) As String

   ' trims delimiter character frin the end of a string
   TrimDelimChar = S
   If Len(S) Then
      If Right$(S, 1) = DELIM_CHAR Then TrimDelimChar = Left$(S, Len(S) - 1)
   End If

End Function

